package infpp.oceanlife;

import java.awt.Point;
import java.io.Serializable;
import java.util.Random;

import javax.swing.ImageIcon;

import static java.lang.Math.*;

/**
 *        Implements objects of the type bubble. The coordinates and the icon
 *        are given from the super class "OceanObject". The movement direction
 *        is defined here.
 * @author tdu, akue
 *  
 */
public class Bubble extends OceanObject {
    

    /**
     * Sets the width of the x-direction of the bubble's movement.
     */
    private int dir0 = 3;
    
    
    /**
     * Sets the amplitude of the x-direction of the bubble's movement.
     */
    private int dir1 = -5;
    
    /**
     * Sets the width of the y-direction of the bubble's movement. 
     */
    private int dir2 = -8;
    
    /**
     * Sets the amplitude of the y-direction of the bubble's movement. 
     */
    private int dir3 = 3;

    /**
     * @param x
     *            the bubble's x coordinate
     * @param y
     *            the bubble's y coordinate
     * @param bubbleIcon
     *            the bubble's image icon
     * @param currentOcean
     *            The ocean the bubble resides in.           
     * @throws throws exception for the position
     */
    public Bubble(int x, int y, ImageIcon bubbleIcon, Ocean currentOcean)
	    throws Exception {
	
    	super(x, y, bubbleIcon, currentOcean);
	
    }

    @Override
    /**
	 * Updates the bubble's direction vector
	 * 		   Switch statement for the cases when the bubble collides with
	 *         the ocean bounds. 
	 * 		   The bubble is removed when it collides with the upper bound.
	 * 		   Exception for removing the bubble is caught here.
	 */
    protected void updateDirection() {

	this.direction[0] = (int) (dir0 + dir1 * random());
	this.direction[1] = (int) (dir2 + dir3 * random());

	switch (objectCollidesWithOceanBounds()) {

	case COLLISION_LEFT_BOUND:
	    dir0 = abs(dir0);
	    dir1 = -abs(dir1);
	    direction[0] = 0;
	    break;

	case COLLISION_RIGHT_BOUND:
	    dir0 = -abs(dir0);
	    dir1 = abs(dir1);
	    direction[0] = 0;
	    break;

	case COLLISION_LOWER_BOUND:
	    dir2 = -abs(dir2);
	    dir3 = abs(dir3);
	    direction[1] = 0;
	    break;

	case COLLISION_UPPER_BOUND:
	    dir2 = 0;
	    dir3 = 0;
	    
	    direction[0] = 0;
	    direction[1] = 0;
	    
	    
	    
	    
	    try {
		synchronized (currentOcean.getOceanObjects() )
		
		{currentOcean.removeOceanObject
			(currentOcean.getOceanObjects().indexOf(this));
		}
		
	    } catch (Exception e) {
		// TODO Auto-generated catch block
		System.out.println
		("couldn't remove bubble from water surface");
	    }
	    
	    
	    break;

	case NO_COLLISION:
	    break;
	}

    }

    /**
	 * @see infpp.oceanlife.OceanObject#toString()
	 * 
	 * @return The subclass name "Bubble" and the height of the Icon is
	 * printed; includes the super class "toString".
	 */
    @Override
    public String toString() {
	// TODO Auto-generated method stub

	System.out.println(this.getObjectIcon().getIconHeight());
	return "Bubble" + super.toString();
    }
}
