package infpp.oceanlife;

import static java.lang.Math.*;
import java.awt.*;
import java.io.Serializable;

import javax.swing.ImageIcon;

/**
 * 
 * Implements objects of the type fish. The coordinates and the icon are
 *        given from the super class "OceanObject". The movement direction is
 *        defined here.
 * @author tdu, akue
 * @param dir0
 *            Sets the width of the x-direction of the fish's movement.
 * @param dir1
 *            Sets the amplitude of the x-direction of the fish's movement.
 * @param dir2
 *            Sets the width of the y-direction of the fish's movement.
 * @param dir3
 *            Sets the amplitude of the y-direction of the fish's movement.
 */
public class Fish extends OceanObject {
	/**
	 * the icon to use for the fish
	 */
	public ImageIcon fishIcon;
	
	
	/**
     * Sets the width of the x-direction of the bubble's movement.
     */
    private int dir0 = 7;
    
    
    /**
     * Sets the amplitude of the x-direction of the bubble's movement.
     */
    private int dir1 = -4;
    
    /**
     * Sets the width of the y-direction of the bubble's movement. 
     */
    private int dir2 = -3;
    
    /**
     * Sets the amplitude of the y-direction of the bubble's movement. 
     */
    private int dir3 = -4;

	/**
	 * @param x
	 *            the fish's x coordinate
	 * @param y
	 *            the fish's y coordinate
	 * @param fishIcon
	 *            the fish's image icon
	 * @param currentOcean
	 *            the ocean the fish resides in
	 * @throws throws exception for the position
	 */
	public Fish(int x, int y, ImageIcon fishIcon, Ocean currentOcean)
			throws Exception {

		super(x, y, fishIcon, currentOcean);

	}

	@Override
	/**
	 * Updates the direction of the fish's movement.
	 * 		   Switch statement for the cases when the fish collides with the
	 *         ocean bounds. The fish changes its image icon if it collides
	 *         with either the left or the right bound and changes its
	 *         direction into the opposite. 
	 */
	protected void updateDirection() {

		ImageIcon fishIcon;

		this.direction[0] = (int) (dir0 + dir1 * random());
		this.direction[1] = (int) (dir2 + dir3 * random());

		switch (objectCollidesWithOceanBounds()) {

		case COLLISION_LEFT_BOUND:
			dir0 = abs(dir0);
			dir1 = -abs(dir1);
			direction[0] = 0;

			if (this.isHighlighted()) {
				fishIcon = new ImageIcon(getClass().getResource(
						currentOcean.getHighlightedFishIconRightwardLocation()));
			} else {
				fishIcon = new ImageIcon(getClass().getResource(
						currentOcean.getFishIconRightwardLocation()));
			}
			fishIcon.setDescription("rightward");
			this.setObjectIcon(fishIcon);
			break;

		case COLLISION_RIGHT_BOUND:
			dir0 = -abs(dir0);
			dir1 = abs(dir1);
			direction[0] = 0;

			if (this.isHighlighted()) {
				fishIcon = new ImageIcon(getClass().getResource(
						currentOcean.getHighlightedFishIconLeftwardLocation()));
			} else {
				fishIcon = new ImageIcon(getClass().getResource(
						currentOcean.getFishIconLeftwardLocation()));				
			}
			fishIcon.setDescription("leftward");
			this.setObjectIcon(fishIcon);
			break;

		case COLLISION_LOWER_BOUND:
			dir2 = -abs(dir2);
			dir3 = abs(dir3);
			direction[1] = 0;
			break;

		case COLLISION_UPPER_BOUND:
			dir2 = abs(dir2);
			dir3 = -abs(dir3);
			direction[1] = 0;
			break;

		case NO_COLLISION:
			break;
		}

	}

	/* (non-Javadoc)
	 * @see infpp.oceanlife.OceanObject#toString()
	 * 
	 * @return The subclass name "Fish" and the height of the Icon is printed,
	 *         including the super class "toString".
	 */
	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return "Fish" + super.toString();
	}

	/**
	 * @return Returns the fishIcon.
	 */
	public ImageIcon getFishIcon() {
		return fishIcon;
	}

	
	/**
	 * @param fishIcon the fish icon to set
	 */
	public void setFishIcon(ImageIcon fishIcon) {
		this.fishIcon = fishIcon;
	}
}
