/**
 * 
 */
package infpp.oceanlife;

import java.util.LinkedList;
import java.util.concurrent.Semaphore;
import java.awt.*;
import java.io.Serializable;

import javax.print.attribute.standard.NumberOfDocuments;
import javax.swing.ImageIcon;

/**
 * Implements the constructor of oceans, the ocean objects and the addition and
 * removal of ocean objects.
 * 
 * @author tdu, akue
 * 
 */
public class Ocean implements OceanInterface, Serializable {

	/**
	 * The width of the ocean.
	 */
	private int width;

	/**
	 * The width of the ocean.
	 */
	private int depth;

	/**
	 * The sleep time for ocean object threads.
	 */
	private int animationSpeed = 30;

	/**
	 * The linked list that holds the information about ocean objects.
	 */
	final private LinkedList<OceanObject> oceanObjects = new LinkedList<OceanObject>();

	/**
	 * The ocean background colour
	 */
	private Color oceanBackgroundColor = new Color(0x1428D2);

	/**
	 * Indicates whether an oil catastrophe is happening.
	 */
	private boolean oilCatastrophe = false;

	/**
	 * The position of the last mouse click
	 */
	private Point ClickPosition = new Point(-1, -1);

	/**
	 * Gives the number of currently highlighted objects.
	 */
	private int numberOfHighlightedObjects = 0;

	/**
	 * Indicates whether the GUI thread is currently running.
	 */
	private boolean oceanGUIRunning = false;

	/**
	 * The types of ocean object available
	 */
	private String[] OceanObjectStrings = { "FISH", "STONE", "BUBBLE", "PLANT",
			"OIL_BLOB" };

	/**
	 * The location of the icon to use if the fish swims leftward and is not
	 * highlighted.
	 */
	private String FishIconLeftwardLocation = "/infpp/oceanlife/images/fish_leftward.png";

	/**
	 * The location of the icon to use if the fish swims rightward and is not
	 * highlighted.
	 */
	private String FishIconRightwardLocation = "/infpp/oceanlife/images/fish_rightward.png";

	/**
	 * The location of the icon to use if the fish swims leftward and is
	 * highlighted.
	 */
	private String HighlightedFishIconLeftwardLocation = "/infpp/oceanlife/images/fish_leftward_highlighted.png";

	/**
	 * The location of the icon to use if the fish swims rightward and is
	 * highlighted.
	 */
	private String HighlightedFishIconRightwardLocation = "/infpp/oceanlife/images/fish_rightward_highlighted.png";

	/**
	 * The location of the icon to use for oil blobs.
	 */
	private String OilBlobLocation = "/infpp/oceanlife/images/Oil_drop.png";

	/**
	 * @return The location of the icon to use if the fish swims leftward and is
	 *         not highlighted.
	 */
	public String getFishIconLeftwardLocation() {
		return FishIconLeftwardLocation;
	}

	/**
	 * @return The location of the icon to use if the fish swims rightward and
	 *         is not highlighted.
	 */
	public String getFishIconRightwardLocation() {
		return FishIconRightwardLocation;
	}

	/**
	 * @return The location of the icon to use if the fish swims leftward and is
	 *         highlighted.
	 */
	public String getHighlightedFishIconLeftwardLocation() {
		return HighlightedFishIconLeftwardLocation;
	}

	/**
	 * @return The location of the icon to use if the fish swims rightward and
	 *         is not highlighted.
	 */
	public String getHighlightedFishIconRightwardLocation() {
		return HighlightedFishIconRightwardLocation;
	}

	/**
	 * @param fishIconLeftwardLocation
	 *            The location of the icon to use if the fish swims leftward and
	 *            is not highlighted.
	 */
	public void setFishIconLeftwardLocation(String fishIconLeftwardLocation) {
		FishIconLeftwardLocation = fishIconLeftwardLocation;
	}

	/**
	 * @param fishIconRightwardLocation
	 *            The location of the icon to use if the fish swims rightward
	 *            and is not highlighted.
	 */
	public void setFishIconRightwardLocation(String fishIconRightwardLocation) {
		FishIconRightwardLocation = fishIconRightwardLocation;
	}

	/**
	 * @param highlightedFishIconLeftwardLocation
	 *            The location of the icon to use if the fish swims leftward and
	 *            is highlighted.
	 */
	public void setHighlightedFishIconLeftwardLocation(
			String highlightedFishIconLeftwardLocation) {
		HighlightedFishIconLeftwardLocation = highlightedFishIconLeftwardLocation;
	}

	/**
	 * @param highlightedFishIconRightwardLocation
	 *            The location of the icon to use if the fish swims rightward
	 *            and is highlighted.
	 */
	public void setHighlightedFishIconRightwardLocation(
			String highlightedFishIconRightwardLocation) {
		HighlightedFishIconRightwardLocation = highlightedFishIconRightwardLocation;
	}

	/**
	 * The location of the icon to use for stones.
	 */
	private String StoneIconLocation = "/infpp/oceanlife/images/stone.png";

	/**
	 * The location of the icon to use for highlighted stones.
	 */
	private String HighlightedStoneIconLocation = "/infpp/oceanlife/images/stone_highlighted.png";

	/**
	 * The location of the icon to use for bubbles.
	 */
	private String BubbleIconLocation = "/infpp/oceanlife/images/bubble.png";

	/**
	 * The location of the icon to use for highlighted bubbles.
	 */
	private String HighlightedBubbleIconLocation = "/infpp/oceanlife/images/bubble_highlighted.png";

	/**
	 * The location of the icon to use for plants.
	 */
	private String PlantIconLocation = "/infpp/oceanlife/images/plant.png";

	/**
	 * The location of the icon to use for highlighted plants.
	 */
	private String HighlightedPlantIconLocation = "/infpp/oceanlife/images/plant_decay.png";

	/**
	 * @return The location of the icon to use for oil blobs.
	 */
	public String getOilBlobLocation() {
		return OilBlobLocation;
	}

	/**
	 * @param oilBlobLocation
	 *            The location of the icon to use for oil blobs.
	 */
	public void setOilBlobLocation(String oilBlobLocation) {
		OilBlobLocation = oilBlobLocation;
	}

	/**
	 * @param width
	 *            width of the ocean
	 * @param depth
	 *            depth of the ocean
	 * @param oceanObjects
	 *            objects in the ocean
	 */
	public Ocean(int width, int depth, LinkedList<OceanObject> oceanObjects)
			throws Exception {
		// TODO Auto-generated constructor stub
		// hier wird der Ozean gefuellt

		if (width <= 0 || depth <= 0) {
			throw new Exception("negative values for ocean");
		} else {
			setWidth(width);
			setDepth(depth);
			initialiseOceanObjects(oceanObjects);
		}
	}

	/**
	 * @return Returns a string of the status of the ocean including width,
	 *         depth and objects.
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		// TODO Auto-generated method stub
		String oceanState = new String();

		oceanState = oceanState + "The ocean has a width of " + this.width
				+ " and a depth of " + this.depth
				+ " units\nand contains the following objects:\n\n";

		/** traverse the list of ocean objects, if it is not empty */
		if (oceanObjects.size() == 0) {

			oceanState = oceanState + "[none]\n\n----------\n";

		} else {

			for (OceanObject obj : oceanObjects) {
				oceanState = oceanState + obj.toString();
			}

			oceanState = oceanState + "\n----------\n";
		}

		return oceanState;

	}

	/**
	 * @return Depth of the ocean.
	 * @see infpp.oceanlife.OceanInterface#getDepth()
	 */
	public int getDepth() {
		// TODO Auto-generated method stub
		return this.depth;
	}

	/**
	 * @return Returns true if the thread of the OceanGUI is running.
	 */
	public boolean isOceanGUIRunning() {
		return oceanGUIRunning;
	}

	/**
	 * @param oceanRunning
	 *            a boolean indicating whether or not the GUI should run.
	 */
	public void setOceanGUIRunning(boolean oceanRunning) {
		this.oceanGUIRunning = oceanRunning;
	}

	/**
	 * @return the animationSpeed
	 */
	public int getAnimationSpeed() {
		return animationSpeed;
	}

	/**
	 * @param animationSpeed
	 *            the animationSpeed to set
	 */
	public void setAnimationSpeed(int animationSpeed) {
		this.animationSpeed = animationSpeed;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see infpp.oceanlife.OceanInterface#getWidth()
	 */
	/**
	 * @return width of the ocean
	 */
	public int getWidth() {
		// TODO Auto-generated method stub
		return this.width;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see infpp.oceanlife.OceanInterface#setdepth(int)
	 */
	/**
	 * @param depth
	 *            the depth of the ocean to set
	 * @throws exception
	 *             for negative value
	 */
	public void setDepth(int depth) throws Exception {
		// TODO Auto-generated method stub
		// if(depth < 0){
		// throw new IllegalArgumentException("Negative depth");
		// }
		this.depth = depth;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see infpp.oceanlife.OceanInterface#setWidth(int)
	 */
	/**
	 * @param width
	 *            set the width of the ocean
	 * @throws exception
	 *             for negative value
	 */
	public void setWidth(int width) {
		// TODO Auto-generated method stub
		// if(width < 0){
		// throw new IllegalArgumentException("Negative width");
		// }
		this.width = width;
	}

	/**
	 * @return the oceanObjectStrings
	 */
	public String[] getOceanObjectStrings() {
		return OceanObjectStrings;
	}

	/**
	 * @param oceanObjectStrings
	 *            the oceanObjectStrings to set
	 */
	public void setOceanObjectStrings(String[] oceanObjectStrings) {
		OceanObjectStrings = oceanObjectStrings;
	}

	/**
	 * @return the oceanBackgroundColor
	 */
	public Color getOceanBackgroundColor() {
		return oceanBackgroundColor;
	}

	/**
	 * @param oceanBackgroundColor
	 *            the oceanBackgroundColor to set
	 */
	public void setOceanBackgroundColor(Color oceanBackgroundColor) {
		this.oceanBackgroundColor = oceanBackgroundColor;
	}

	/**
	 * @return a boolean indicating whether an oil catastrophe is happening.
	 */
	public boolean isOilCatastrophe() {
		return oilCatastrophe;
	}

	/**
	 * @param oilCatastrophe
	 *            a boolean signalling an impending oil catastrophe
	 */
	public void setOilCatastrophe(boolean oilCatastrophe) {
		this.oilCatastrophe = oilCatastrophe;
	}

	/**
	 * @return The location of the icon to use for stones.
	 */
	public String getStoneIconLocation() {
		return StoneIconLocation;
	}

	/**
	 * @return The location of the icon to use for highlighted stones.
	 */
	public String getHighlightedStoneIconLocation() {
		return HighlightedStoneIconLocation;
	}

	/**
	 * @return The location of the icon to use for bubbles.
	 */
	public String getBubbleIconLocation() {
		return BubbleIconLocation;
	}

	/**
	 * @return The location of the icon to use for highlighted bubbles.
	 */
	public String getHighlightedBubbleIconLocation() {
		return HighlightedBubbleIconLocation;
	}

	/**
	 * @return The location of the icon to use for plants.
	 */
	public String getPlantIconLocation() {
		return PlantIconLocation;
	}

	/**
	 * @return The location of the icon to use for highlighted plants.
	 */
	public String getHighlightedPlantIconLocation() {
		return HighlightedPlantIconLocation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see infpp.oceanlife.OceanInterface#getOceanObjects()
	 */
	/**
	 * @return the objects in the ocean
	 */
	public LinkedList<OceanObject> getOceanObjects() {
		// TODO Auto-generated method stub

		return this.oceanObjects;
	}

	/**
	 * @return the position of the last mouse click
	 */
	public Point getClickPosition() {
		return ClickPosition;
	}

	/**
	 * @param clickPosition the
	 *            position of the last mouse click as reported by the GUI.
	 */
	public void setClickPosition(Point clickPosition) {
		ClickPosition = clickPosition;
	}

	/**
	 * @return Returns true if the position of the ocean object is valid insofar
	 *         as it is inside the ocean. Collision detection is not done here.
	 */
	public boolean isPositionValid(OceanObject obj) {
		int x;
		int y;
		x = obj.getPosition()[0];
		y = obj.getPosition()[1];
		return (x >= 0 && y >= 0 && x <= width && y <= depth);
	}

	/**
	 * @param toAdd
	 *            an oceanObject that should be added to the ocean.
	 * @return Returns true if the ocean object collides with another ocean
	 *         object.
	 */
	public boolean collisionDetected(OceanObject toAdd) {

		boolean xOK = true;
		boolean yOK = true;

		int newXPos = toAdd.getPositionX();
		int newWidth = toAdd.getDimensionsOfIcon().width;

		int newYPos = toAdd.getPositionY();
		int newHeight = toAdd.getDimensionsOfIcon().height;

		if (oceanObjects.size() > 0) {

			for (OceanObject alreadyThere : oceanObjects) {

				int oldXPos = alreadyThere.getPositionX();
				int oldWidth = alreadyThere.getDimensionsOfIcon().width;
				int oldYPos = alreadyThere.getPositionY();
				int oldHeight = alreadyThere.getDimensionsOfIcon().height;

				if (!((newXPos < oldXPos && newXPos + newWidth < oldXPos) || ((newXPos > oldXPos
						+ oldWidth)))) {

					xOK = false;

				}
				if (!((newYPos < oldYPos && newYPos + newHeight < oldYPos) || ((newYPos > oldYPos
						+ oldHeight)))) {

					yOK = false;

				}

				if (xOK == false && yOK == false) {
					return !false;
				} else {
					xOK = true;
					yOK = true;
				}

			}

		}

		if ((newXPos + newWidth > width) || (newYPos + newHeight > depth)
				|| (newXPos < 0) || (newYPos < 0)) {
			return !false;
		} else {
			return !true; // (xOK && yOK);
		}
	}

	/**
	 * @param oceanObjects
	 *            Sets ocean objects.
	 * @throws IllegalArgumentException
	 *             Throws exceptions for invalid ocean object positions.
	 */
	public void initialiseOceanObjects(LinkedList<OceanObject> oceanObjects)
			throws Exception {
		// TODO Auto-generated method stub

		boolean objectsOK = true;
		for (OceanObject obj : oceanObjects) {

			if (!isPositionValid(obj))
				objectsOK = false;
			//
			else {
				addOceanObject(obj);
				//
			}
		}

		if (!objectsOK) {
			throw new IllegalArgumentException("Initialisation failed");

		}

		/** create a thread for each object in the ocean */

		for (OceanObject obj : oceanObjects) {

			Thread t = new Thread(obj);
			t.start();

		}

	}

	/**
	 * @return Returns the listIndex (in the linked list oceanObjects) of the
	 *         ocean object on the given point in the ocean.
	 */
	public int getObjectIndexFromClickPosition(Point p) {

		int listIndex = -1;

		LinkedList<OceanObject> objectList = oceanObjects;

		for (OceanObject obj : objectList) {

			if ((p.x >= obj.getPositionX() && p.x <= obj.getPositionX()
					+ obj.getDimensionsOfIcon().width)
					&& (p.y >= obj.getPositionY() && p.y < obj.getPositionY()
							+ obj.getDimensionsOfIcon().height)) {
				listIndex = objectList.indexOf(obj);

			}

		}

		return listIndex;
	}

	/**
	 * @param obj
	 *            the ocean object whose type we wish to know
	 * @return an integer specifying the object type
	 */
	public int getClassIndex(OceanObject obj) {
		String type = obj.getClass().getSimpleName();

		int numericType = -1;

		for (int k = 0; k < OceanObjectStrings.length; k++) {

			if (type.equalsIgnoreCase(OceanObjectStrings[k])) {
				numericType = k;

			}
		}
		return numericType;
	}

	/**
	 * @param obj
	 *            the object to highlight or unhighlight.
	 */
	public void highlight(OceanObject obj) {

		obj.setHighlighted(!obj.isHighlighted());

		// Character type = obj.getClass().getSimpleName().charAt(0);
		// System.out.println(type.digit(type, 10));

		// Typ des angeklickten Objekts bestimmen

		int numericType = getClassIndex(obj);

		if (obj.isHighlighted()) {
			numberOfHighlightedObjects++;

			switch (numericType) {

			case 0: // FISH

				if (obj.getObjectIcon().getDescription().equalsIgnoreCase(
						"rightward")) { // the fish swims to the right
					ImageIcon fishIcon = new ImageIcon(getClass().getResource(
							HighlightedFishIconRightwardLocation));

					fishIcon.setDescription("rightward");
					obj.setObjectIcon(fishIcon);
				} else { // the fish swims to the left

					ImageIcon fishIcon = new ImageIcon(getClass().getResource(
							HighlightedFishIconLeftwardLocation));

					fishIcon.setDescription("leftward");

					obj.setObjectIcon(fishIcon);

				}

				break;

			case 1: // STONE

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						HighlightedStoneIconLocation)));

				break;

			case 2: // BUBBLE

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						HighlightedBubbleIconLocation)));

				break;

			case 3: // PLANT

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						HighlightedPlantIconLocation)));

			}

		} else {

			numberOfHighlightedObjects--;

			switch (numericType) {

			case 0: // FISH

				if (obj.getObjectIcon().getDescription().equalsIgnoreCase(
						"rightward")) {
					ImageIcon fishIcon = new ImageIcon(getClass().getResource(
							FishIconRightwardLocation));

					fishIcon.setDescription("rightward");
					obj.setObjectIcon(fishIcon);
				} else { // der Fisch schwimmt nach links

					ImageIcon fishIcon = new ImageIcon(getClass().getResource(
							FishIconLeftwardLocation));

					fishIcon.setDescription("leftward");

					obj.setObjectIcon(fishIcon);

				}
				break;

			case 1: // STONE

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						StoneIconLocation)));

				break;

			case 2: // BUBBLE

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						BubbleIconLocation)));

				break;

			case 3: // PLANT

				obj.setObjectIcon(new ImageIcon(getClass().getResource(
						PlantIconLocation)));

			}

			/**
			 * Oil blobs cannot be highlighted. Being able to remove them would
			 * take all the fun out of the game.
			 */

		}

	}

	/**
	 * @return the number of highlighted objects
	 */
	public int getNumberOfHighlightedObjects() {
		return numberOfHighlightedObjects;
	}

	/**
	 * @param numberOfHighlightedObjects
	 *            the number of highlighted objects to set, should this be
	 *            necessary in the future.
	 */
	public void setNumberOfHighlightedObjects(int numberOfHighlightedObjects) {
		this.numberOfHighlightedObjects = numberOfHighlightedObjects;
	}

	/* (non-Javadoc)
	 * @see infpp.oceanlife.OceanInterface#addOceanObject(infpp.oceanlife.OceanObject)
	 */
	public void addOceanObject(OceanObject obj) throws Exception {
		if (collisionDetected(obj) == false) {
			// muss hier noch ein Aufruf von setPosition stehen?

			oceanObjects.add(obj);

		} else {
			throw new IllegalArgumentException("illegal object position");
		}
	}

	/* (non-Javadoc)
	 * @see infpp.oceanlife.OceanInterface#removeOceanObject(int)
	 */
	public void removeOceanObject(int x) throws Exception {
		if (x > getOceanObjects().size() - 1) {
			throw new IllegalArgumentException("Illegal index");

		} else {
			// Objekt aus der Liste loeschen
			getOceanObjects().remove(x);

			// den zugehoerigen Thread nicht hier killen,
			// sondern von der GUI aus

			// System.out.println("Object was removed successfully.");
		}

	}

}
