/**
 * 
 */
package infpp.oceanlife;

import javax.swing.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;

import javax.swing.event.MouseInputAdapter;
import static java.lang.Math.*;

/**
 * 
 * OceanGUI implements the graphic user interface (including the ocean?)
 * @author tdu, akue
 */


public class OceanGUI extends JFrame implements Runnable {

    /**
     * This method makes sure that all of the window is repainted, not only the
     * water panel.
     */
    public void myRepaint() {
	this.repaint();
    }

    /**
     * the ocean we work with
     */
    Ocean localOcean;

    /**
     * the dropdown box to select what kind of OceanObject to add
     */
    JComboBox objectsToAdd;

    /**
     * the TextField to specify the x coordinate of a new object's position
     */
    TextField xEntry = new TextField(3);

    /**
     * the TextField to specify the y coordinate of a new object's position
     */
    TextField yEntry = new TextField(3);

    /**
     * the string to hold the user's input to xEntry
     */
    String xValue;

    /**
     * the string to hold the user's input to yEntry
     */
    String yValue;

    /**
     * the position of the last mouse click, regardless of button
     */
    Point ClickPosition = new Point(-1, -1);

    /**
     * the position where a drag with the mouse originated
     */
    Point dragPositionStart = new Point(-1, -1);

    /**
     * the position where a drag with the mouse ended
     */
    Point dragPositonEnd = new Point(-1, -1);

    /**
     * the current position of the pointer
     */
    Point mousePointerPosition = new Point(-1, -1);

    /**
     * says whether fire mode is on, which adds objects in rapid succession
     */
    private boolean fireMode = false;

    /**
     * the rectangle most recently selected
     */
    Rectangle selectedRectangle = new Rectangle();

    /**
     * the rectangle selected before the current selection was made
     */
    Rectangle oldRectangle = new Rectangle();

    /**
     * the button that adds objects at the specified position
     */
    JButton addButton = new JButton("Add");

    /**
     * the button that removes highlighted objects
     */
    JButton removeButton = new JButton("Remove");

    /**
     * the button that starts the fun!
     */
    JButton startButton = new JButton("Start");

    /**
     * the button that steps through the motion
     */
    JButton stepButton = new JButton("Step");

    /**
     * the button that stops the motion
     */
    JButton stopButton = new JButton("Stop");

    /**
     * the button that loads an ocean from a file
     */
    JButton loadButton = new JButton("Load");

    /**
     * the button that saves an ocean to a file
     */
    JButton saveButton = new JButton("Save");

    /**
     * the button that prints the ocean's current population
     */
    JButton printButton = new JButton("Print");

    /**
     * Press this button should you ever want to leave the ocean.
     */
    JButton quitButton = new JButton("Quit");

    /**
     * Does what it says on the box: Displays the ocean and all of its
     * inhabitants.
     */
    JPanel water = new JPanel(false) {
	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	public void paintComponent(Graphics g) {

	    super.paintComponent(g);

	    if (localOcean.isOilCatastrophe() == false) {
		drawOcean(g);
	    } else {
		drawOilCatastrophe(g);
		localOcean.setOilCatastrophe(false);
	    }
	}
    };

    /**
     * the panel where all the controls go.
     */
    JPanel buttonPanel = new JPanel(false);

    /**
     * Constructs a new window to display an ocean. Takes a window title and the
     * ocean in question.
     * 
     * @param title
     *            The window title
     * @param oceanToDisplay
     *            The ocean to work with
     */
    /**
     * @param title
     * @param oceanToDisplay
     */
    /**
     * @param title
     * @param oceanToDisplay
     */
    OceanGUI(String title, Ocean oceanToDisplay) {

	super(title);
	this.localOcean = oceanToDisplay;

	setLayout(new BorderLayout(0, 20));

	objectsToAdd = new JComboBox(localOcean.getOceanObjectStrings());

	JLabel x = new JLabel("x");
	JLabel y = new JLabel("y");

	water.setSize(localOcean.getWidth(), localOcean.getDepth());

	FocusAdapter xEntryKeyboardFocusListener = new FocusListenerEx();
	FocusAdapter yEntryKeyboardFocusListener = new FocusListenerEx();

	/**
	 * we wrote our own key listener since the built-in ones don't work with
	 * panels reliably.
	 */
	defineSelfMadeKeyListener();

	xEntry.addFocusListener(xEntryKeyboardFocusListener);
	yEntry.addFocusListener(yEntryKeyboardFocusListener);

	quitButton.addActionListener(new QuitButtonActionListenerEx());
	addButton.addActionListener(new AddButtonActionListenerEx());
	removeButton.addActionListener(new RemoveButtonActionListenerEx());
	printButton.addActionListener(new PrintButtonActionListenerEx());
	stepButton.addActionListener(new StepButtonActionListenerEx());
	stopButton.addActionListener(new StopButtonActionListenerEx());
	startButton.addActionListener(new StartButtonActionListenerEx());
	saveButton.addActionListener(new SaveButtonActionListenerEx());
	loadButton.addActionListener(new LoadButtonActionListenerEx());

	MouseInputAdapter waterListener = new MouseInputListenerEx();

	water.addMouseListener(waterListener);
	water.addMouseMotionListener(new MouseMotionListenerEx());
	addMouseWheelListener(new MouseWheelListenerEx());

	buttonPanel.add(objectsToAdd);
	buttonPanel.add(x);
	buttonPanel.add(xEntry);
	buttonPanel.add(y);
	buttonPanel.add(yEntry);
	buttonPanel.add(addButton);
	buttonPanel.add(startButton);
	buttonPanel.add(stepButton);
	buttonPanel.add(stopButton);
	buttonPanel.add(removeButton);
	buttonPanel.add(loadButton);
	buttonPanel.add(saveButton);
	buttonPanel.add(printButton);
	buttonPanel.add(quitButton);

	buttonPanel.setLayout(new FlowLayout());

	add(buttonPanel, BorderLayout.NORTH);
	add(water, BorderLayout.CENTER);

	/**
	 * We're a bit at a loss here. The automatic sizing doesn't work as
	 * expected, so we had to tweak at the numbers manually. We know this is
	 * bad style, but we wanted the addOceanObject and similar methods to
	 * work.
	 */
	super.setSize(10 + max(buttonPanel.getWidth(), water.getWidth()), 92
		+ buttonPanel.getHeight() + water.getHeight());

	super.setDefaultCloseOperation(EXIT_ON_CLOSE);

	/**
	 * Another cheap way to hide imperfections in the size calculations.
	 */
	super.setResizable(false);

	water.setVisible(true);
	buttonPanel.setVisible(true);

	this.setLocation(100, 20);

	/**
	 * we don't want the dropdown to have the focus
	 */
	objectsToAdd.setFocusable(false);

	setVisible(true);

	Thread t = new Thread(this);
	t.start();

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     * 
     * Why does it say "non-Javadoc" here? - Anyway, this is the method that
     * keeps the GUI alive, essentially by repainting it every five
     * milliseconds.
     */
    public void run() {
	while (true) {

	    try {
		Thread.sleep(5);
	    } catch (InterruptedException e) {
		// TODO Auto-generated catch block
		System.out.println("Sleeping failed");
	    }

	    updatePanelStatus();

	    if (localOcean.isOceanGUIRunning()) {
		if (fireMode) {
		    try {
			Thread.sleep(100);
		    } catch (InterruptedException e) {
			// TODO Auto-generated catch block
			System.out.println("Sleeping failed");
		    }
		    fireOneObject();
		}
		this.repaint();
	    }
	}
    }

    /**
     * This method is called periodically by the run() method above and checks
     * which buttons should be clickable and which should not.
     */
    private void updatePanelStatus() {

	/** if the ocean is empty, not much can be done */
	if (localOcean.getOceanObjects().size() == 0) {

	    /** first of all, the motion should be stopped */
	    localOcean.setOceanGUIRunning(false);

	    /** the add button should always be clickable */

	    startButton.setEnabled(false);
	    stepButton.setEnabled(false);
	    stopButton.setEnabled(false);

	    /**
	     * the remove button is treated in the removeOceanObject method
	     */

	    loadButton.setEnabled(true);
	    saveButton.setEnabled(false); // no use saving an empty ocean

	    /**
	     * the print button should always be clickable
	     * 
	     * ditto for the quit button. even if nothing else works, quitting
	     * should be possible :-)
	     */

	} else { // if the ocean is not empty

	    if (localOcean.isOceanGUIRunning()) {
		// the ocean is not empty, and it is running
		// oh. all code is gone from here. hmmm.

	    } else { // the ocean is not empty, but is is not running
		startButton.setEnabled(true);
		stopButton.setEnabled(false);
		stepButton.setEnabled(true);
		loadButton.setEnabled(true);
		saveButton.setEnabled(true);
	    }
	}

	/**
	 * if one of the text fields is empty, clicking the add button is rather
	 * useless. Therefore disable the button. (It would be very easy to
	 * callthe isGUIInputNumeric(String, String) method here and perform
	 * some preliminary sanity checks on the input. But for the sake of
	 * letting the ocean do the checks -- as we were tod --, we don't do
	 * those checks here.)
	 */

	if ((xEntry.getText().isEmpty()) || (yEntry.getText().isEmpty())) {
	    addButton.setEnabled(false);
	} else {
	    addButton.setEnabled(true);
	}

	/**
	 * the remove button should only be clickable if there are highlighted
	 * objects
	 */
	if (localOcean.getNumberOfHighlightedObjects() > 0) {
	    removeButton.setEnabled(true);
	} else {
	    removeButton.setEnabled(false);
	}

    }

    /**
     * @author tdu This class caters for automagically selecting all the text in
     *         a text field when the user clicks it or tabs into it. It also
     *         unselects everything in it when the user clicks somehere else or
     *         tabs out of it.
     */
    class FocusListenerEx extends FocusAdapter {
	public void focusGained(FocusEvent e) {
	    if (e.getSource().equals(xEntry)) {
		xEntry.selectAll();
	    } else if (e.getSource().equals(yEntry)) {
		yEntry.selectAll();
	    }

	}

	public void focusLost(FocusEvent e) {
	    if (e.getSource().equals(xEntry)) {
		xEntry.select(0, 0);
	    } else if (e.getSource().equals(yEntry)) {
		yEntry.select(0, 0);
	    }
	}

    }

    /**
     * @author tdu This class provides the current position of the pointer,
     *         useful for the fire mode.
     */
    class MouseMotionListenerEx extends MouseMotionAdapter {
	public void mouseMoved(MouseEvent e) {

	    mousePointerPosition = e.getPoint();
	    // System.out.println(mousePointerPosition);
	    // xEntry.setText(Integer.toString(mousePointerPosition.x));
	    // yEntry.setText(Integer.toString(mousePointerPosition.y));

	}
    }

    /**
     * @author tdu This class reacts to turning the mouse wheel, which we use to
     *         set the speed of the animation.
     */
    class MouseWheelListenerEx implements MouseWheelListener {
	public void mouseWheelMoved(MouseWheelEvent e) {

	    int threshold = 1000; // ms
	    int currentDelay = localOcean.getAnimationSpeed();
	    int granularity = 0;

	    if (currentDelay < 20) {
		granularity = 2;
	    } else {
		granularity = currentDelay / 8;
	    }

	    switch (e.getWheelRotation()) {
	    case 1:
		currentDelay += granularity;
		break;
	    case -1:
		currentDelay -= granularity;
		break;
	    }

	    if (currentDelay >= threshold) {
		currentDelay = threshold;
	    } else if (currentDelay <= 0) {
		currentDelay = 0;
	    }

	    localOcean.setAnimationSpeed(currentDelay);

	    System.out.println("Current delay: "
		    + localOcean.getAnimationSpeed() + " ms");

	}
    }

    /**
     * @author tdu This class handles all the mouse clicks. Clicking the left,
     *         middle, or right button on an object or on water all invoke
     *         different actions.
     */
    class MouseInputListenerEx extends MouseInputAdapter {
	public void mouseClicked(MouseEvent e) {

	    ClickPosition = e.getPoint();
	    int whichButton = e.getButton();

	    xEntry.setText(Integer.toString(ClickPosition.x));
	    yEntry.setText(Integer.toString(ClickPosition.y));

	    localOcean.setClickPosition(ClickPosition);

	    /**
	     * here we find out which object, if any, has been clicked.
	     */
	    int index = localOcean
		    .getObjectIndexFromClickPosition(ClickPosition);

	    if (index > -1) { // an object was clickd

		OceanObject clickedObject = localOcean.getOceanObjects().get(
			index);

		switch (whichButton) { // 1: left, 2: middle, 3: right
		case 1:
		    localOcean.highlight(clickedObject);
		    break;
		case 2:
		    try {			
			localOcean.removeOceanObject(index);
			clickedObject.setToDie(true);

		    } catch (Exception e1) {
			// TODO Auto-generated catch block
			System.out.println("couldn't remove object");
		    }
		    ;
		    break;
		case 3:
		    /**
		     * This serves to quickly add a different type of object,
		     * and was implemented before we decided to also provide
		     * keyboard accelerators for that.
		     */

		    objectsToAdd.setSelectedIndex(localOcean
			    .getClassIndex(clickedObject));
		    break;
		}

		/**
		 * The object may have been highlighted, therefore we repaint
		 * its area in the ocean.
		 */
		water.repaint(clickedObject.getPositionX(), clickedObject
			.getPositionY(),
			clickedObject.getDimensionsOfIcon().width,
			clickedObject.getDimensionsOfIcon().height);		

	    } else { // no object was clicked
		switch (whichButton) { // 1: left, 2: middle, 3: right
		case 1:
		    break;
		case 2:
		    break;
		case 3: // add the currently selected type of object to the
		        // ocean
		    addObjectToOcean();
		    break;

		}
	    }

	}

	public void mousePressed(MouseEvent e) {
	    if (e.getButton() == 1) {
		/**
		 * if the user starts to drag a rectangle while pointing at
		 * an object, that object's highlighting status is not easy to
		 * determine. therefore we enable dragging only when started
		 * at an empty point in the ocean.
		 */
		
		dragPositionStart = new Point(-1, -1);
		

		if (localOcean.getObjectIndexFromClickPosition(e.getPoint()) == -1) {
		    // the user clicked water
		    dragPositionStart = e.getPoint();
		}

	    }
	}

	public void mouseReleased(MouseEvent e) {
	    

	    /**
	     * If the left button was depressed during the drag, make a
	     * rectangle. If any other button was depressed, do nothing, so as
	     * not to interfere with the clicking of the right button to add
	     * objects and the middle button to remove them.
	     */
	    if (e.getButton() == 1) {
		if (dragPositionStart.x != -1 && dragPositionStart.y != -1) {
		    dragPositonEnd = e.getPoint();

		    /**
		     * normally, we check whether the diagonal of the selected
		     * rectangle is longer than ten pixels so that miniature
		     * rectangles do not get painted. However, as we do not
		     * have enough time to meaningfully implement the
		     * selection and subsequent manipulation (including, but
		     * ideally not limited to, removal) of objects, we set the
		     * threshold to 42000, effectively disabling selection, at
		     * least until we get around to coding it.
		     */

		    int width = abs(dragPositonEnd.x - dragPositionStart.x);
		    int height = abs(dragPositonEnd.y - dragPositionStart.y);
		    if (dragPositionStart.distance(dragPositonEnd) > 10) {

			Point upperLeftCorner = new Point
			(min(dragPositonEnd.x,
			 dragPositionStart.x), min(dragPositonEnd.y,
			 dragPositionStart.y));

			oldRectangle = selectedRectangle;

			selectedRectangle.setRect(upperLeftCorner.x,
				upperLeftCorner.y, width, height);

//			System.out.println(dragPositionStart.x + " "
//				+ dragPositionStart.y + " " + width + " "
//				+ height);


			water.repaint();

		    }
		}
	    }

	}

    }

    /**
     * ActionListener for the GUI: Quit Button
     */
    class QuitButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    System.exit(0);
	}
    }

    /**
     * ActionListener for the GUI: Save Button
     */
    class SaveButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {

	    File filename = null;

	    FileDialog dialog = new FileDialog(OceanGUI.this, "Save Ocean",
		    FileDialog.SAVE);
	    dialog.setVisible(true);

	    if (dialog.getFile() == null) {
		System.out.println("Nothing selected");
	    } else {
		filename = new File(dialog.getFile());

		try {
		    OutputStream os = new FileOutputStream(filename);
		    ObjectOutputStream oos = new ObjectOutputStream(os);

		    oos.writeObject(localOcean);

		    oos.close();
		    System.out.println("File saved successfully");

		} catch (IOException e2) {
		    System.out.println("Couldn't save file");
		}
	    }

	}
    }

    /**
     * ActionListener for the GUI: Load Button
     */
    class LoadButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {

	    File filename = null;

	    FileDialog dialog = new FileDialog(OceanGUI.this, "Load Ocean",
		    FileDialog.LOAD);
	    dialog.setVisible(true);

	    try {
		filename = new File(dialog.getFile());
		InputStream is = new FileInputStream(filename);
		ObjectInputStream ois = new ObjectInputStream(is);
		localOcean = (Ocean) ois.readObject();
		System.out.println(localOcean);

		for (OceanObject blah : localOcean.getOceanObjects()) {		    
		    blah.start();
		}

		System.out.println("File loaded successfully");

		myRepaint();
		ois.close();
	    } catch (Exception e3) {
		System.out.println("Couldn't load file");
	    }
	}
    }

    class StepButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {

	    /**
	     * the step button can only be clicked when the ocean is not
	     * running
	     */

	    for (OceanObject blah : localOcean.getOceanObjects()) {

		Point oldPosition = new Point(blah.getPositionX(), blah
			.getPositionY());

		int[] newPosition = { oldPosition.x + blah.direction[0],
			oldPosition.y + blah.direction[1] };

		try {
		    blah.updateDirection();
		    blah.setPositionWhenMoving(newPosition);

		} catch (Exception e42) {

		    System.out.println("can't move object");
		}

	    }

	    water.repaint();
	}
    }

    class StopButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {

	    localOcean.setOceanGUIRunning(false);
	    stepButton.setEnabled(true);
	    loadButton.setEnabled(true);
	    saveButton.setEnabled(true);
	    startButton.setEnabled(true);
	    water.repaint();
	}
    }

    class StartButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {

	    localOcean.setOceanGUIRunning(true);
	    startButton.setEnabled(false);
	    stepButton.setEnabled(false);
	    stopButton.setEnabled(true);
	    loadButton.setEnabled(false);
	    saveButton.setEnabled(false);

	}
    }

    /**
     * ActionListener for the GUI: Remove Button
     */
    class RemoveButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    
	    /**
	     * traverse the list of ocean objects in reverse order,
	     * so as not to scramble the indexes
	     */	    

	    LinkedList<OceanObject> tempList = localOcean.getOceanObjects();

	    for (int h = tempList.size() - 1; h >= 0; h--) {

		OceanObject blah = tempList.get(h);

		if (blah.isHighlighted()) {

		    try {
			Thread.sleep(30);

			blah.setHighlighted(false);
			localOcean.setNumberOfHighlightedObjects(localOcean
				.getNumberOfHighlightedObjects() - 1);
			localOcean.removeOceanObject(h);
			blah.setToDie(true);

			water.paintImmediately(blah.getPositionX(), blah
				.getPositionY(),
				blah.getDimensionsOfIcon().width, blah
					.getDimensionsOfIcon().height);
		    } catch (Exception s) {
			System.out.println("This will never be shown!");
		    }

		}
	    }

	    /**
	     * if there are no more objects left in the ocean, halt the
	     * (nonexistent) motion.
	     */
	    if (localOcean.getOceanObjects().size() == 0) {
		localOcean.setOceanGUIRunning(false);
	    }
	    
	}
    }

    public void addObjectToOcean() {
	
	xValue = xEntry.getText();
	yValue = yEntry.getText();

	OceanObject newObject = null;

	if (!isGUIInputNumeric(xValue, yValue)) {
	    throw new IllegalArgumentException("Only integers allowed");
	}

	int selectedObjectIndex;

	/**
	 * we can't really explain why we had to do this distinction.
	 * The class OilBlob inherits directly from OceanObject,
	 * therefore it should work just like the others. *gnarg*
	 */
	if (localOcean.isOilCatastrophe()) {

	    selectedObjectIndex = -1;

	} else {

	    selectedObjectIndex = objectsToAdd.getSelectedIndex();

	}

	switch (selectedObjectIndex) {

	case 0: // FISH

	    try {
		

		ImageIcon fishIcon = new ImageIcon(getClass().getResource(
			localOcean.getFishIconRightwardLocation()));

		fishIcon.setDescription("rightward");

		newObject = new Fish(Integer.parseInt(xValue), Integer
			.parseInt(yValue),

		fishIcon, localOcean);

		Thread t = new Thread(newObject);
		t.start();

		localOcean.addOceanObject(newObject);

	    } catch (Exception e1) {
		// TODO Auto-generated catch block
		System.out.println("Couldn't add Fish");
		// e1.printStackTrace();
	    }
	    ;
	    break;

	case 1:

	    try {
		newObject = new Stone(Integer.parseInt(xValue), Integer
			.parseInt(yValue),

		new ImageIcon(getClass().getResource(
			localOcean.getStoneIconLocation())), localOcean);

		Thread t = new Thread(newObject);
		t.start();

		localOcean.addOceanObject(newObject);

	    } catch (Exception e1) {
		// TODO Auto-generated catch block
		System.out.println("Couldn't add Stone");
	    }
	    ;
	    break;

	case 2:

	    try {
		newObject = new Bubble(Integer.parseInt(xValue), Integer
			.parseInt(yValue),

		new ImageIcon(getClass().getResource(
			localOcean.getBubbleIconLocation())), localOcean);

		Thread t = new Thread(newObject);
		t.start();

		localOcean.addOceanObject(newObject);

	    } catch (Exception e1) {
		// TODO Auto-generated catch block
		System.out.println("Couldn't add Bubble");
	    }
	    ;
	    break;

	case 3:

	    try {

		ImageIcon plantIcon = new ImageIcon(getClass().getResource(
			localOcean.getPlantIconLocation()));

		newObject = new Plant(Integer.parseInt(xValue), Integer
			.parseInt(yValue),

		plantIcon, localOcean);

		Thread t = new Thread(newObject);
		t.start();

		localOcean.addOceanObject(newObject);

	    } catch (Exception e1) {
		// TODO Auto-generated catch block
		System.out.println("Couldn't add Plant");
	    }
	    ;
	    break;

	case 4:

	    try {

		ImageIcon oilBlobIcon = new ImageIcon(getClass().getResource(
			localOcean.getOilBlobLocation()));

		newObject = new OilBlob(Integer.parseInt(xValue), Integer
			.parseInt(yValue),

		oilBlobIcon, localOcean);

		Thread t = new Thread(newObject);
		t.start();

		localOcean.addOceanObject(newObject);

	    } catch (Exception e1) {
		// TODO Auto-generated catch block
		System.out.println("Couldn't add Oil Blob");
	    }
	    ;
	    break;

	}

	water.repaint();


    }

    /**
     * ActionListener for the GUI: Add Button
     */
    class AddButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    addObjectToOcean();
	    xEntry.setText("");
	    yEntry.setText("");
	}

    }

    /**
     * ActionListener for the GUI: Print Button
     */
    class PrintButtonActionListenerEx implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    System.out.println(localOcean);
	}

    }

    
    /**
     * @param x The input from the first text field
     * @param y The input from the second text field
     * @return a boolean indicating whether both of the input fields contain
     * numbers
     */
    public boolean isGUIInputNumeric(String x, String y) {

	try {
	    Integer.parseInt(x);
	} catch (NumberFormatException e) {
	    return false;
	}
	try {
	    Integer.parseInt(y);
	} catch (NumberFormatException e) {
	    return false;
	}

	return true;
    }

    
    /**
     * This method adds one object at the current pointer's position. 
     */
    private void fireOneObject() {

	xEntry.setText(Integer.toString(mousePointerPosition.x));
	yEntry.setText(Integer.toString(mousePointerPosition.y));

	if (localOcean.isOilCatastrophe()) {

	} else {
	    addObjectToOcean();
	}

    }

    /**
     * @param g The graphics object to paint with.
     */
    private void drawOcean(Graphics g) {

	
	g.setColor(localOcean.getOceanBackgroundColor());
	
	g.fillRect(0, 0, water.getWidth(), water.getHeight());
	Graphics2D g2d = (Graphics2D) g;

	synchronized (localOcean.getOceanObjects()) {

	    for (OceanObject blah : localOcean.getOceanObjects()) {

		g2d.drawImage(blah.getObjectIcon().getImage(), blah
			.getPositionX(), blah.getPositionY(), water);

	    }
	}

	

	
	/**
	 * here the new rectangle is drawn, if the user dragged one
	 */
	g.setColor(Color.WHITE);
	g.drawRect(selectedRectangle.x, selectedRectangle.y,
		selectedRectangle.width - 1, selectedRectangle.height - 1);

    }

    

    /**
     * Here we specify that the GUI should react to some keyboard input.
     * We only deal with those keys that are of interest to us, rather than
     * intercepting all keyboard input and discarding most of it. 
     */
    private void defineSelfMadeKeyListener() {

	
	InputMap im = getRootPane().getInputMap 
		(JComponent.WHEN_IN_FOCUSED_WINDOW);
	ActionMap am = getRootPane().getActionMap(); // Aktionen ausloesen

	/**
	 * This line says that something should happen when the f key is
	 * pressed.
	 */
	am.put("ActionFor_F_Key", fKeyPressed);
	
	/**
	 * This line contains a reference to the action that should happen
	 * when the f key is pressed.
	 */
	im.put(KeyStroke.getKeyStroke('f'), "ActionFor_F_Key");

	am.put("ActionFor_S_Key", sKeyPressed);
	im.put(KeyStroke.getKeyStroke('s'), "ActionFor_S_Key");

	am.put("ActionFor_B_Key", bKeyPressed);
	im.put(KeyStroke.getKeyStroke('b'), "ActionFor_B_Key");

	am.put("ActionFor_P_Key", pKeyPressed);
	im.put(KeyStroke.getKeyStroke('p'), "ActionFor_P_Key");

	am.put("ActionFor_O_Key", oKeyPressed);
	im.put(KeyStroke.getKeyStroke('o'), "ActionFor_O_Key");

	am.put("ActionFor_!_Key", ExclamationMarkKeyPressed);
	im.put(KeyStroke.getKeyStroke('!'), "ActionFor_!_Key");

    }

    /**
     * Here we specify how the GUI should react to the keyboard input.
     */

    Action fKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {
	    objectsToAdd.setSelectedIndex(0);
	    // now a fish can be added
	}

    };

    Action sKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {
	    objectsToAdd.setSelectedIndex(1);
	    // now a stone can be added
	}

    };

    Action bKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {
	    objectsToAdd.setSelectedIndex(2);
	    // now a bubble can be added
	}

    };

    Action pKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {
	    objectsToAdd.setSelectedIndex(3);
	    // now a plant can be added
	}

    };

    Action oKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {
	    /**
	     * As of 8 July 2010, the oil catastrophe does not work the way we
	     * would like it to, therefore nothing happens here. For a
	     * preview, uncomment the two lines.
	     * UPDATE: It is now possible to simulate an oil catastrophe by
	     * switching to fire mode and adding oil blobs. It should normally
	     * be possible to bind this action to the "o" key, or even to the
	     * keys "b" and "p" pressed in immediate succession, but, alas,
	     * our time and expertise is limited.
	     */
	    // localOcean.setOilCatastrophe(true);
	    // water.repaint();
	}

    };

    Action ExclamationMarkKeyPressed = new AbstractAction() {
	public void actionPerformed(ActionEvent e) {

	    fireMode = !fireMode;

	}

    };

    /**
     * Here the oil catastrophe is (partially) implemented.
     * @param g The graphics object to work with
     */

    private void drawOilCatastrophe(Graphics g) {

	fireMode = true;

	xValue = xEntry.getText();
	yValue = yEntry.getText();

	OceanObject newObject = null;

	try {
	    newObject = new OilBlob(Integer.parseInt(xValue), Integer
		    .parseInt(yValue),

	    new ImageIcon(getClass().getResource(
		    localOcean.getOilBlobLocation())), localOcean);

	    Thread t = new Thread(newObject);
	    t.start();
	    localOcean.setOceanGUIRunning(true);

	    localOcean.addOceanObject(newObject);

	} catch (Exception e1) {
	    // TODO Auto-generated catch block
	    System.out.println("Couldn't add OilBlob");
	}
	;

    }

}
