package infpp.oceanlife;

import static java.lang.Math.abs;
import static java.lang.Math.random;

import java.io.Serializable;

import javax.swing.ImageIcon;

/**
 * 
 * Implements objects of the type plant. The coordinates and the icon are
 *        given from the super class "OceanObject". The movement direction is
 *        defined here.
 * @author tdu, akue
 */
public class Plant extends OceanObject {
	
    
	/**
     * Sets the width of the x-direction of the bubble's movement.
     */
    private int dir0 = 2;
    
    
    /**
     * Sets the amplitude of the x-direction of the bubble's movement.
     */
    private int dir1 = -4;
    
    /**
     * Sets the width of the y-direction of the bubble's movement. 
     */
    private int dir2 = 1;
    
    /**
     * Sets the amplitude of the y-direction of the bubble's movement. 
     */
    private int dir3 = 8;
	
	
	
	/**
	 * @param x
	 *            the plant' x coordinate
	 * @param y
	 *            the plant's y coordinate
	 * @param plantIcon
	 *            the plant's image icon
	 * @param currentOcean
	 *            the ocean the plant resides in
	 * @throws throws exception for the position
	 */
	public Plant(int x, int y, ImageIcon plantIcon, Ocean currentOcean) throws Exception {
		
		super(x, y, plantIcon, currentOcean);
		
	}

	
	    @Override
	    /**
		 * Updates the direction of the plant's movement.
		 * 		   Switch statement for the cases when the plant collides with
		 *         the ocean bounds. The plant stops moving when it
		 *         collides with the lower bound.
		 */
	    protected void updateDirection() {

		this.direction[0] = (int) (dir0 + dir1 * random());
		this.direction[1] = (int) (dir2 + dir3 * random());

		switch (objectCollidesWithOceanBounds()) {

		case COLLISION_LEFT_BOUND:
		    dir0 = abs(dir0);
		    dir1 = -abs(dir1);
		    direction[0] = 0;
		    break;

		case COLLISION_RIGHT_BOUND:
		    dir0 = -abs(dir0);
		    dir1 = abs(dir1);
		    direction[0] = 0;
		    break;

		case COLLISION_LOWER_BOUND:
		    dir2 = 0;
		    dir3 = 0;
		    direction[0] = 0;
		    direction[1] = 0;
		    break;

		case COLLISION_UPPER_BOUND:
		    break;

		case NO_COLLISION:
		    break;
		}

	    }
	    
	    
	    /**
		 * @see infpp.oceanlife.OceanObject#toString()
		 * @return The subclass name "Plant" and the height of the Icon is
		 *         printed; includes the super class "toString".
		 */
	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return "Plant" + super.toString();
	}
}
