package infpp.oceanlife;

import static java.lang.Math.abs;
import static java.lang.Math.random;

import java.io.Serializable;

import javax.swing.ImageIcon;

/**
 * 
 * Implements objects of the type stone. The coordinates and the icon are
 *        given from the super class "OceanObject". The movement direction is
 *        defined here.
 * @author tdu, akue
 */
public class Stone extends OceanObject {
	
    
    /**
     * Sets the width of the x-direction of the bubble's movement.
     */
    private int dir0 = 0;
    
    
    /**
     * Sets the amplitude of the x-direction of the bubble's movement.
     */
    private int dir1 = 1;
    
    /**
     * Sets the width of the y-direction of the bubble's movement. 
     */
    private int dir2 = 1;
    
    /**
     * Sets the amplitude of the y-direction of the bubble's movement. 
     */
    private int dir3 = 12;
		    

	/**
	 * @param x
	 *            the stone's x coordinate
	 * @param y
	 *            the stone's y coordinate
	 * @param stoneIcon
	 *            the stone's image icon
	 * @param currentOcean
	 *            the ocean the stone resides in
	 * @throws throws exception for the position
	 */
	public Stone(int x, int y, ImageIcon stoneIcon, Ocean currentOcean) throws Exception {
		super(x, y, stoneIcon, currentOcean);
		
	}

	    @Override
	    /**
		 * Updates the direction of the stone's movement.
		 * 		   Switch statement for the cases when the stone collides with the ocean bounds. 
		 * 		   The stone stops its movement when it collides with the lower bound.
		 */
	    protected void updateDirection() {

		this.direction[0] = (int) (dir0 + dir1 * random());
		this.direction[1] = (int) (dir2 + dir3 * random());

		switch (objectCollidesWithOceanBounds()) {

		case COLLISION_LEFT_BOUND:
		    dir0 = abs(dir0);
		    dir1 = -abs(dir1);
		    direction[0] = 0;
		    break;

		case COLLISION_RIGHT_BOUND:
		    dir0 = -abs(dir0);
		    dir1 = abs(dir1);
		    direction[0] = 0;
		    break;

		case COLLISION_LOWER_BOUND:
		    dir0 = 0;
		    dir1 = 0;
		    dir2 = 0;
		    dir3 = 0;
		    direction[0] = 0;
		    direction[1] = 0;
		    break;

		case COLLISION_UPPER_BOUND:
		    break;

		case NO_COLLISION:
		    break;
		}

	    }
	    
	
	    /**
		 * @see infpp.oceanlife.OceanObject#toString()
		 * @return The subclass name "Stone" and the height of the Icon is printed;
		 *         includes the super class "toString".
		 */
	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return "Stone" + super.toString();
	}
}
